@@ -1,4 +1,4 @@
 test_that("simu_db() returns a dataframe", {
-  simu_db() %>% inherits('data.frame')
+  simu_db() %>% expect_s3_class('data.frame')
 })
 
